var http = require('http'),
    path = require('path'),
    url = require('url'),
    request = require('request'),
    _ = require('lodash'),
    jschardet = require('jschardet'),
    jsdom = require('jsdom'),
    zlib = require("zlib"),
    fs = require("fs"),
    Pool = require('generic-pool').Pool;

// Fallback on iconv-lite if we didn't succeed compiling iconv
// https://github.com/sylvinus/node-crawler/pull/29
var iconv, iconvLite;
try {
    iconv = require('iconv').Iconv;
} catch (e) {}

if (!iconv) {
    iconvLite = require('iconv-lite');
}

function useCache(options) {
    return (options.uri && (options.cache || options.skipDuplicates) && (options.method=="GET" || options.method=="HEAD"));
}

function checkJQueryNaming (options) {
    if (typeof options.jquery !== 'undefined' && typeof options.jQuery === 'undefined') {
        options.jQuery = options.jquery;
        delete options.jquery;
    }
    return options;
}

Crawler.prototype.init = function (options) {
    var self = this;

    // you can use jquery or jQuery
    options = checkJQueryNaming(options);

    // if using rateLimits we want to use only one connection with delay in between requests
    if (options.rateLimits !== 0) {
        options.maxConnections = 1;
    }

    //Default options
    self.options = _.extend({
        autoWindowClose:    true,
        cache:              false, //false,true, [ttl?]
        forceUTF8:          false,
        incomingEncoding:   null, //TODO remove or optimize
        jQuery:             true,
        jQueryUrl:          path.resolve(__dirname,"../vendor/jquery-1.8.3.min.js"),
        maxConnections:     10,
        method:             "GET",
        onDrain:            false,
        priority:           5,
        priorityRange:      10,
        referer:            false,
        retries:            3,
        retryTimeout:       10000,
        skipDuplicates:     false,
        onDrain:            false,
        rateLimits:         0,
        referer:            false,
        incomingEncoding:   null
    },options);

    // Don't make these options persist to individual queries
    var globalOnlyOptions = ["maxConnections", "priorityRange", "onDrain"];

    //Setup a worker pool w/ https://github.com/coopernurse/node-pool
    self.pool = Pool({
        name         : 'crawler',
        //log        : self.options.debug,
        max          : self.options.maxConnections,
        priorityRange: self.options.priorityRange,
        create       : function(callback) {
            callback(1);
        },
        destroy      : function(client) {}
    });

    self.plannedQueueCallsCount = 0;
    self.queueItemSize = 0;

    self.onDrain = function() {};
    self.cache = {};
};

Crawler.prototype._release = function(options) {
    var self = this;

    self.queueItemSize--;
    if (options._poolReference) {
        self.pool.release(options._poolReference);
    }

    // Pool stats are behaving weird - have to implement our own counter
    if (self.queueItemSize + self.plannedQueueCallsCount === 0) {
        if (self.options.onDrain) {
            self.options.onDrain();
        }
    }
};

Crawler.prototype.queue = function (options) {
    var self = this;

    //Did I get a list ? Queue all the URLs.
    if (_.isArray(options)) {
        var queueLength = options.length
        for (var i = 0; i < queueLength; i++) {
            self.queue(options[i]);
        }
        // Did I get a simple string instead, convert to object and queue
    } else if (_.isString(options)) {
        self._addToQueue({
            uri: options
        });
    } else {
        self._addToQueue(options);
    }
};

Crawler.prototype._addToQueue = function (options) {
    var self = this;
    self.queueItemSize++;

    // you can use jquery or jQuery
    options = checkJQueryNaming(options);

    _.defaults(options, self.options);

    // Remove all the global options from our options
    // TODO we are doing this for every _addToQueue, find a way to avoid this
    _.each(self.globalOnlyOptions, function(globalOnlyOption) {
        delete options[globalOnlyOption];
    });

    // If duplicate skipping is enabled, avoid queueing entirely for URLs we already crawled
    if (options.skipDuplicates && self.cache[options.uri]) {
        return self._release(options);
    }

    // acquire connection - callback function is called
    // once a resource becomes available
    self.pool.acquire(function(error, poolReference) {
        options._poolReference = poolReference;

        // this is and operation error
        if (error) {
            console.error("pool acquire error:",error);
            options.callback(error);
            //TODO UGLY
            return;
        }

        //Static HTML was given, skip request
        if (options.html) {
            self._onContent(null, options, {body:options.html}, false);
        } else if (typeof options.uri === "function") {
            options.uri( function(uri) {
                options.uri = uri;
                self._makeRequest(options);
            });
        } else {
            self._makeRequest(options);
        }
    }, options.priority);
};

Crawler.prototype._makeRequest = function (options) {
    var self = this;

    if (typeof options.rateLimits === 'number' && options.rateLimits !== 0) {
        setTimeout(function() {
            self._executeRequest(options);
        }, options.rateLimits);
    } else {
        self._executeRequest(options);
    }    
};

Crawler.prototype._executeRequest = function (options) {
    var self = this;

    if (useCache(options)) {

        var cacheData = self.cache[options.uri];

        //If a query has already been made to self URL, don't callback again
        if (cacheData) {

            // Make sure we actually have cached data, and not just a note
            // that the page was already crawled
            if (_.isArray(cacheData)) {
                self._onContent(null, options, cacheData[0], true);
            } else {
                self._release(options);
            }
        }
    }

    if (options.debug) {
        console.log(options.method+" "+options.uri+" ...");
    }
    
    // Cloning keeps the opts parameter clean:
    // - some versions of "request" apply the second parameter as a
    // property called "callback" to the first parameter
    // - keeps the query object fresh in case of a retry
    // Doing parse/stringify instead of _.clone will do a deep clone and remove functions

    var ropts = JSON.parse(JSON.stringify(options));

    if (!ropts.headers) ropts.headers={};
    if (ropts.forceUTF8) {
        if (!ropts.headers["Accept-Charset"] && !ropts.headers["accept-charset"]) ropts.headers["Accept-Charset"] = 'utf-8;q=0.7,*;q=0.3';
        if (!ropts.encoding) ropts.encoding=null;
    }
    if (typeof ropts.encoding === 'undefined') {
        ropts.headers["Accept-Encoding"] = "gzip";
        ropts.encoding = null;
    }
    if (ropts.userAgent) {
        ropts.headers["User-Agent"] = ropts.userAgent;
    }
    if (ropts.referer) {
        ropts.headers["Referer"] = ropts.referer;
    }
    if (ropts.proxies && ropts.proxies.length) {
        ropts.proxy = ropts.proxies[0];
    }

    var requestArgs = ["uri","url","qs","method","headers","body","form","json","multipart","followRedirect","followAllRedirects",
        "maxRedirects","encoding","pool","timeout","proxy","auth","oauth","strictSSL","jar","aws"];


    var req = request(_.pick.apply(this,[ropts].concat(requestArgs)), function(error,response,body) {
        if (error) return self._onContent(error, options);

        response.uri = response.request.href;

        // Won't be needed after https://github.com/mikeal/request/pull/303 is merged
        if (response.headers['content-encoding'] && response.headers['content-encoding'].toLowerCase().indexOf('gzip') >= 0) {
            zlib.gunzip(response.body, function (error, body) {
                if (error) return self.onContent(error, options);

                if (!options.forceUTF8) {
                    response.body = body.toString(req.encoding);
                } else {
                    response.body = body;
                }

                self._onContent(error,options,response,false);
            });
        } else {
            self._onContent(error,options,response,false);
        }

    });
};

Crawler.prototype._onContent = function (error, options, response, fromCache) {
    var self = this;

    if (error) {
        if (options.debug) {
            console.log("Error "+error+" when fetching "+options.uri+(options.retries?" ("+options.retries+" retries left)":""));
        }
        if (options.retries) {
            self.plannedQueueCallsCount++;
            setTimeout(function() {
                options.retries--;
                self.plannedQueueCallsCount--;

                // If there is a "proxies" option, rotate it so that we don't keep hitting the same one
                if (options.proxies) {
                    options.proxies.push(options.proxies.shift());
                }

                self.queue(options);
            },options.retryTimeout);

        } else if (options.callback) {
            options.callback(error);
        }

        return self._release(options);
    }

    if (!response.body) response.body="";

    if (options.debug) {
        console.log("Got "+(options.uri||"html")+" ("+response.body.length+" bytes)...");
    }

    if (options.forceUTF8) {
        //TODO check http header or meta equiv?
        if (!options.incomingEncoding) {
            var detected = jschardet.detect(response.body);

            if (detected && detected.encoding) {
                if (options.debug) {
                    console.log("Detected charset " + detected.encoding + " (" + Math.floor(detected.confidence * 100) + "% confidence)");
                }
                if (detected.encoding != "utf-8" && detected.encoding != "ascii") {

                    if (iconv) {
                        var iconvObj = new iconv(detected.encoding, "UTF-8//TRANSLIT//IGNORE");
                        response.body = iconvObj.convert(response.body).toString();

                        // iconv-lite doesn't support Big5 (yet)
                    } else if (detected.encoding != "Big5") {
                        response.body = iconvLite.decode(response.body, detected.encoding);
                    }

                } else if (typeof response.body != "string") {
                    response.body = response.body.toString();
                }

            } else {
                response.body = response.body.toString("utf8"); //hope for the best
            }
        } else { // do not hope to best use custom encoding
            if (iconv) {
                var iconvObj = new iconv(options.incomingEncoding, "UTF-8//TRANSLIT//IGNORE");
                response.body = iconvObj.convert(response.body).toString();
                // iconv-lite doesn't support Big5 (yet)
            } else if (options.incomingEncoding != "Big5") {
                response.body = iconvLite.decode(response.body, options.incomingEncoding);
            }
        }

    } else {
        response.body = response.body.toString();
    }

    if (useCache(options) && !fromCache) {
        if (options.cache) {
            self.cache[options.uri] = [response];

            //If we don't cache but still want to skip duplicates we have to maintain a list of fetched URLs.
        } else if (options.skipDuplicates) {
            self.cache[options.uri] = true;
        }
    }

    if (!options.callback) return self._release(options);

    response.options = options;

    // This could definitely be improved by *also* matching content-type headers
    var isHTML = response.body.match(/^\s*</);

    if (isHTML && options.jQuery && options.method!="HEAD") {

        // TODO support for non-HTML content
        // https://github.com/joshfire/node-crawler/issues/9
        try {
            var jsd = function(src) {
                var env = jsdom.env({
                    "url":response.uri,
                    "html":response.body,
                    "src":src,
                    "done":function(errors,window) {

                        var callbackError = false;

                        try {
                            response.window = window;
                            options.callback(errors,response,window.jQuery);
                        } catch (e) {
                            callbackError = e;
                        }

                        // Free jsdom memory
                        if (options.autoWindowClose) {
                            try {
                                window.close();
                                window = null;
                            } catch (err) {
                                console.log("Couldn't window.close : "+err);
                            }
                            response.body = null;
                            response = null;
                        }

                        self._release(options);

                        if (callbackError) throw callbackError;
                    }
                });
            };

            // jsdom doesn't support adding local scripts,
            // We have to read jQuery from the local fs
            if (options.jQueryUrl.match(/^(file\:\/\/|\w+\:|\/)/)) {

                // TODO cache this
                fs.readFile(options.jQueryUrl.replace(/^file\:\/\//,""),"utf-8",function(err,jq) {
                    if (err) {
                        options.callback(err);
                        self._release(options);
                    } else {
                        try {
                            jsd([jq]);
                        } catch (e) {
                            options.callback(e);
                            self._release(options);
                        }
                    }
                });
            } else {
                jsd([options.jQueryUrl]);
            }
        } catch (e) {
            options.callback(e);
            self._release(options);
        }
    } else {

        options.callback(null,response);
        self._release(options);
    }
}

function Crawler(options) {
    var self = this;
    self.init(options);
}

module.exports = Crawler;
module.exports.VERSION = '0.3.0';